/*****************************************************************************
 * Copyright (c) 2012 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: d00_pm_mitigation_crd4.sas
 *
 * PURPOSE: Create methods for credit risk mitigation rules.
 *
 *   USAGE: 
 *
 *   NOTES: 
 *          
 *
 *****************************************************************************/

%macro d00_pm_mitigation_crd4;

%macro adj_std;
    %if "&APPLY_CCF_AFTER_CRM" = "YES" %then %do;
      if OFF_BALANCE_SHEET_FLG=1 then 
         _VALUE_ = max(0, sum(EAD, -SPECIFIC_PROVISION*CCF/100));
      else
         _VALUE_ = max(0, sum(EAD, -SPECIFIC_PROVISION));
    %end;
    %else %do;
        _VALUE_ = max(0, sum(EAD, -SPECIFIC_PROVISION));
    %end;
%mend adj_std;

%macro zero_ccf;
   %if "&APPLY_CCF_AFTER_CRM" = "YES" %then %do;
   /*- The following is only necessary for a zero ccf case -*/
      if MIT_CCF_FLG = 1 then do;
         used_value = min(_mitigant_adjval_, OUTV_MIT_VALUE);
         OUTV_MIT_VALUE = OUTV_MIT_VALUE - used_value;
      end;
      else used_value = 0;
   %end;
   %else %do;
      used_value=0;
   %end;
%mend zero_ccf;

proc compile outlib  = RD_TEMP.&RD_TEMPLATE
             inlib   = RD_TEMP.&RD_TEMPLATE
             package = regulatory_rw
             label   = "Regulatory Risk Weight Methods"
             env     = RD_TEMP.&RD_TEMPLATE ; 
             
/*------------------------------------------------------------------------------
 *
 * BEGIN: method for exposures in the CRM allocation project
 *
 *------------------------------------------------------------------------------
 */
  
method PM_REGULATORY_EXP kind = price
                         desc = 'Valuation: Exposures to be covered';

 /*- Basel II - 52: Under STD, exposures should be risk-weighted net of specific provision -*/ 
 /*- CRR Article 111(1): The exposure value shall be its accounting value remaining after specific credit risk adjustments, -*/
 /*- additional value adjustments and other own funds reductions related to the asset item have been applied.               -*/
    if APPROACH = 'STD' then do;
       %adj_std;
    end;
    else 
       _value_=EAD;
       
    OUTV_VALUE = EXP_VALUE;
    OUTV_EAD = EAD;
    OUTV_RWA_NO_CRM = _value_ * RW_EXP / 100;   
    OUTV_CAPITAL_NO_CRM = OUTV_RWA_NO_CRM * &CAPITAL_RATIO;
    OUTV_MIT_VALUE = EXP_VALUE;    

    
    OUTV_RWA_SECURED = 0;
    OUTV_RWA_UNSECURED = OUTV_RWA_NO_CRM;
    
    OUTV_CRM_USED_AMT = 0;
    OUTV_CRM_USED_AMT_EFF = 0;
    
    OUTV_EXPOSURE_PROVISION = sum(0, SPECIFIC_PROVISION);
    OUTV_EL_AMT = EAD * EL / 100; 
    
    /*- only meaningful in the mitigant data -*/
    OUTV_MIT_RWA = 0;
    OUTV_MIT_USED_AMT_EFF = 0;
    OUTV_MIT_USED_AMT = 0;
    
    OUTV_LGD = LGD_EXP;
    OUTV_PD  = PD_EXP;
    
    LGD_WEIGHTED_SUM = LGD_EXP * EAD;
    PD_WEIGHTED_SUM = PD_EXP * EAD; 
    
    HAIRCUT_ADJ_FLG = 0;
    
endmethod;                            
                             
/*------------------------------------------------------------------------------
 *
 * END: method for exposures in the CRM allocation project
 *
 *------------------------------------------------------------------------------
 */


/*------------------------------------------------------------------------------
 *
 * BEGIN: method for credit risk mitigant in the CRM allocation project
 *
 *------------------------------------------------------------------------------
 */
 
method PM_REGULATORY_CRM kind = price
                         desc = 'Valuation: CRM to be allocated';
                       
   _value_ = CRM_VALUE;
   
endmethod;

/*------------------------------------------------------------------------------
 *
 * END: method for credit risk mitigant in the CRM allocation project
 *
 *------------------------------------------------------------------------------
 */


/*------------------------------------------------------------------------------
 *
 * BEGIN:  method credit risk mitigation effect on Regulatory Capital Requirement
 *
 *------------------------------------------------------------------------------
 */
 
method PM_MITIGATION kind = mitigate
                     desc = 'Mitigation Rules on Regulatory Capital' ;
                     
   /*- if the CRM weight is higher than RW then no CRM should be used -*/
   /*- if exposure is zero then no CRM should be used either          -*/
   if _adjexposure_ = 0 or _mult_=0 then do;
      %zero_ccf;
      offset_ead      = 0;
      effective_value = 0;
      secured_rwa     = 0;
      offset_rwa      = 0;
   end;
   
   else do;
     temp = sum(1, MIT_HAIRCUT_EXP/100);
     used_value = min(_mitigant_adjval_, _adjexposure_ / _mult_);
     offset_ead = used_value * _mult_;
     _adjexposure_ = sum(_adjexposure_, -offset_ead); 
     
     /* add the following to get EAD_ADJ. This is needed for CVA calculations */
     OUTV_EAD_ADJ = _adjexposure_ ;
      
     if CRMTYPE in ('FINANCIAL_COLLATERAL', 'NETTING') and CRM_COMP_FLG = 1 then do; 

        /* inflate the effective covered value */
        effective_value = offset_ead * temp;        
        secured_rwa = effective_value * _WEIGHT_ / 100;
        offset_rwa = -effective_value * MIT_RW_EXP /100;
        if HAIRCUT_ADJ_FLG = 0 and used_value > 0 then do;
           /* need to flag the inflation of exposure in RWA computation */
           HAIRCUT_ADJ_FLG = 1;           
           /* adjust EAD_ADJ for financial collaterals. */
           OUTV_EAD_ADJ = _adjexposure_ * temp ; 
           OUTV_RWA_UNSECURED = max(0, sum(OUTV_RWA_UNSECURED*temp, offset_rwa));
        end;
        else 
           /* already inflated */
           OUTV_RWA_UNSECURED = max(0, sum(OUTV_RWA_UNSECURED, offset_rwa));   
     end;
     
     else do; 
        effective_value = offset_ead;
        secured_rwa = effective_value * _WEIGHT_ / 100;
        offset_rwa = - effective_value *  MIT_RW_EXP /100;
        if HAIRCUT_ADJ_FLG = 1 then
           OUTV_RWA_UNSECURED = max(0, sum(OUTV_RWA_UNSECURED, offset_rwa*temp));
        else 
           OUTV_RWA_UNSECURED = max(0, sum(OUTV_RWA_UNSECURED, offset_rwa));
     end;     
   end;
   
   OUTV_CRM_USED_AMT = sum(OUTV_CRM_USED_AMT, used_value);
   
   OUTV_CRM_USED_AMT_EFF = sum(OUTV_CRM_USED_AMT_EFF, effective_value);
   
   OUTV_MIT_RWA = secured_rwa;
   OUTV_MIT_USED_AMT_EFF = effective_value;
   OUTV_MIT_USED_AMT = used_value;
   
   _mitigant_adjval_ = sum(_mitigant_adjval_, - used_value);

   OUTV_RWA_SECURED = sum(OUTV_RWA_SECURED, secured_rwa);
   
   if EXP_APPROACH =: 'IRB' then do;
      
      if CRMTYPE in ('GUARANTEE', 'CREDIT_DERIVATIVE', 'INSURANCE') then do;
        PD_WEIGHTED_SUM = sum(PD_WEIGHTED_SUM, sum(MIT_PD_CRM_CPTY, - MIT_PD_EXP) * offset_ead);
        if "&IRB_USE_GUARANTEE_LGD" = "YES" and MIT_LGD_CRM NE . then 
           LGD_WEIGHTED_SUM = sum(LGD_WEIGHTED_SUM, sum(MIT_LGD_CRM, - MIT_LGD_EXP) * offset_ead) ;        
      end;
   
      else
        LGD_WEIGHTED_SUM = sum(LGD_WEIGHTED_SUM, sum(MIT_LGD_CRM, - MIT_LGD_EXP) * offset_ead) ;
      
   end;
   
endmethod;

/*------------------------------------------------------------------------------
 *
 * END:  method credit risk mitigation effect on Regulatory Capital Requirement
 *
 *------------------------------------------------------------------------------
 */
 
/*------------------------------------------------------------------------------
 *
 * BEGIN:  postvar method for total Regulatory Capital Requirement after mitigation
 *
 *------------------------------------------------------------------------------
 */ 
 
method PVM_CAPITAL kind=postvar
                    desc='Postvar Method after Mitigation' ;
/*- following computation is only valid for exposures -*/                  
    if POSITION_CATEGORY NE 'CRM' then do;    
       OUTV_RWA_PV = OUTV_RWA_SECURED.value + OUTV_RWA_UNSECURED.value;
   if OUTV_EAD.value eq 0 then OUTV_RW = .;
      else OUTV_RW = OUTV_RWA_PV / OUTV_EAD.value * 100;
   if OUTV_EAD.value eq 0 then OUTV_RW_NO_CRM = .;
      else OUTV_RW_NO_CRM = OUTV_RWA_NO_CRM.value / OUTV_EAD.value * 100;
       OUTV_CAPITAL_PV = OUTV_RWA_PV * &CAPITAL_RATIO;
    if OUTV_EAD.value eq 0 then OUTV_PD_EFF_PV = .;
      else OUTV_PD_EFF_PV = PD_WEIGHTED_SUM.value / OUTV_EAD.value; 
    if OUTV_EAD.value eq 0 then OUTV_LGD_EFF_PV = .;
      else OUTV_LGD_EFF_PV = LGD_WEIGHTED_SUM.value / OUTV_EAD.value; 
       OUTV_EL_EFF = OUTV_PD_EFF_PV * OUTV_LGD_EFF_PV / 100;
       OUTV_EL_EFF_AMT_PV = OUTV_VALUE.value * OUTV_EL_EFF / 100;
    end;
    
    else do;
       OUTV_RWA_PV = 0;
       OUTV_RW = 0;
       OUTV_RW_NO_CRM = 0;
    
       OUTV_CAPITAL_PV = 0;
    
       OUTV_PD_EFF_PV = 0; 
       OUTV_LGD_EFF_PV = 0;     
    end;
    
    /* the following variable needs post-processing */ 
    OUTV_VALUE_CON          = .;
    OUTV_EAD_CON            = .;
    OUTV_RWA_CON            = .;
    OUTV_RWA_NO_CRM_CON     = .;
    OUTV_CAPITAL_CON        = .;
    OUTV_CAPITAL_NO_CRM_CON = .;
endmethod;

/*------------------------------------------------------------------------------
 *
 * END:  postvar method for total Regulatory Capital Requirement after mitigation
 *
 *------------------------------------------------------------------------------
 */ 

run;
%mend d00_pm_mitigation_crd4;
