/*****************************************************************************
 * Copyright (c) 2012 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: e15_project_capital_crd4.sas
 *
 * PURPOSE: Calculate RWA and capital results according to the given 
 *          hierarchies
 *
 *
 *   USAGE:  
 *
 *   NOTES: This project calculates RWA and capital results. The user option
 *          MITIGANTS_ALLOCATION_METHOD controls the choice of the following
 *          two projects:
 *          (1) If MITIGANTS_ALLOCATION_METHOD = 'BY_RANK' then
 *              User supplied mitigation rank is used to allocate the credit
 *              risk mitigants (CRMs) to the exposures.  This is done in proc 
 *              RISK before the results are aggregated.
 *          (2) If MITIGANTS_ALLOCATION_METHOD = 'OPTIMAL' then
 *              The CRMs are optimally allocated to the exposures to minimize
 *              the capital requirement.  This is done before this step by using
 *              the network optimization technique.  The results are simply
 *              aggregated here.
 *          
 *
 *****************************************************************************/


%macro e15_project_capital_crd4;
%local macropath ; 
%criskm_get_swc_property(propertyName=criskmmvac.ucmacros.config.path ,q_rd_swc=&RD_SWC,outvar=macropath);

/*- remember output directory -*/
%global ROLLUPPATH;
 
    %local ALLOC_TYPE;

    %if %upcase(&MITIGANTS_ALLOCATION_METHOD) EQ BY_RANK %then 
        %let ALLOC_TYPE = rnk;
    %else
        %let ALLOC_TYPE = opt;

%if &CRM_ALLOCATION_ACROSS_ENTITIES EQ YES %then 
    %let ROLLUPPATH = %sysget(CRISKMRES)&PATHSEP.%lowcase(&REGULATORY_PROJECT_SET_NAME)&PATHSEP.&PROJECT_SET._&ALLOC_TYPE;
%else
    %let ROLLUPPATH = %sysget(CRISKMRES)&PATHSEP.&PROJECT_SET&PATHSEP.&PROJECT_SET._&ALLOC_TYPE;;


    /*- clean the output directory -*/    
    %clean_dir(&ROLLUPPATH);         
  
    %RUNTHEPROJECT:
    
    /*** BEGIN : allocation by rank and aggregate result ***/

    %if %upcase(&MITIGANTS_ALLOCATION_METHOD) NE BY_RANK %then %goto RUNSIMPLEROLLUP;

/* Check for OLAP and configure is necessary */
    %olapreport (&ALLOC_TYPE);

/******************************************/
data work.tmp_exp_crm_link(drop=ccf LGD_CRM_ADJUSTED_FLG CRM_SUBTYPE %if "&APPLY_CCF_AFTER_CRM"= "YES" %then %do; OFF_BALANCE_SHEET_FLG %end;);
set "&REGPATH&PATHSEP.exp_crm_link";
run;
/******************************************/

    proc risk noseqcheck ;
    environment open = RD_PROJ.&ENV_NAME;
/* Include olap options based on user options */
    %inc OLAP;                
    CurExposure MitigationByRank;
                    
    instdata All_exp_capital_&PROJECT_SET file="&REGPATH&PATHSEP.all_exp_incl_sec_underly" format=simple
             type=REGULATORY_EXP
             variables=(EXP_VALUE
                        EAD 
                        RW_EXP  
                        LGD_EXP 
                        PD_EXP 
                        EXP_ID=INSTID
                        SPECIFIC_PROVISION
                        %if "&APPLY_CCF_AFTER_CRM" = "YES" %then %do;
                        OFF_BALANCE_SHEET_FLG 
                        CCF
                        %end;
                        &ALL_CC_VARS
                        );
                                           
    instdata All_crm_capital_&PROJECT_SET file="&REGPATH&PATHSEP.all_crm_input" format=simple
             type=REGULATORY_CRM;
                                      
/******************************************/     
    mitigationdata Exp_Crm_link_&PROJECT_SET file="work.tmp_exp_crm_link"
/******************************************/
                   label="Exposure Credit Risk Mitigant Link"
                   variables=(_MULT_ 
                              MIT_HAIRCUT_EXP 
                              _WEIGHT_ 
                              EXP_APPROACH 
                              CRMTYPE 
                              MIT_PD_CRM 
                              MIT_PD_CRM_CPTY
                              MIT_LGD_CRM
                              MIT_PD_EXP 
                              MIT_LGD_EXP
                              MIT_RW_EXP
                              EXPOSUREID
                              MITIGANTID
                              MITIGATIONTYPE
                              CRM_COMP_FLG
                              %if "&APPLY_CCF_AFTER_CRM" = "YES" %then %do;
                              MIT_CCF_FLG
                              %end;                              
                             )
                             ;
                    
    sources All_capital_&PROJECT_SET All_exp_capital_&PROJECT_SET 
                                     All_crm_capital_&PROJECT_SET 
                                     Exp_Crm_link_&PROJECT_SET;
    
    read sources=All_capital_&PROJECT_SET out=All_capital_&PROJECT_SET outpath="&OUTPATH";
    
    project capital_&PROJECT_SET 
            analysis     = MitigationByRank
            portfolio    = All_capital_&PROJECT_SET
            rundate      = &RUNDATE
            currency     = &REPORT_CCY
            crossclass   = rollup_hierarchy_pos
            outpath      = "&OUTPATH"
            out          = &PROJECT_SET._&ALLOC_TYPE
            postvarmethod= (PVM_CAPITAL)
            report       = (Post_Mitigation_0)

            ;
    
    environment save noupdatemetadata;

    run;
    
    proc risk noseqcheck;
    
    environment open =RD_PROJ.&ENV_NAME;

    /*setpath "%sysget(CMHOME)&pathsep.sascode" < save; */
    /*setpath "&macropath" < save; */
    setpath "&macropath&pathsep.&reg_context" < save;  

    setoptions keeplib;
  
    runproject capital_&PROJECT_SET        
               options = (outall noallscore nomddbs)
               date    = &RUNDATE
               report
               ;
    
    environment save noupdatemetadata;
 
    run;

%r00_post_mitigation_0_crd4 ;

    /*-post-processing -*/
    %post_capital_run;

    %LET cname=%SYSFUNC(CAT(Summary,_,capital_&PROJECT_SET));

    %goto EXIT;
    
    /*** END : allocation by rank and aggregate result ***/    
    
    /*** BEGIN : optimal allocation done and simply aggregate result ***/
    
    %RUNSIMPLEROLLUP:

/* Check for OLAP and configure is necessary */
    %olapreport (&ALLOC_TYPE);

    proc risk noseqcheck ;

    environment open = RD_PROJ.&ENV_NAME;
                     
/* Include olap options based on user options */
    %inc OLAP; 
                             
    marketdata Current_Rollup_&PROJECT_SET file="&REGPATH&PATHSEP.mkt_rollup" type=current;

    instdata All_Rollup_&PROJECT_SET file="&REGPATH&PATHSEP.all_exposures_before_rollup" 
               format=simple type=ROLLUP
               variables=(EXP_ID=INSTID);

    sources All_Rollup_&PROJECT_SET All_Rollup_&PROJECT_SET;

    read sources=All_Rollup_&PROJECT_SET out=All_Rollup_&PROJECT_SET outpath="&OUTPATH";

    
    project Rollup_&PROJECT_SET 
            data          = Current_Rollup_&PROJECT_SET
            portfolio     = All_Rollup_&PROJECT_SET
            rundate       = &RUNDATE
            currency      = &REPORT_CCY
            crossclass    = rollup_hierarchy
            outpath       = "&OUTPATH"
            out           = &PROJECT_SET._&ALLOC_TYPE
            postvarmethod = (PVM_ROLLUP)
            ;

    environment save noupdatemetadata;
    
    run;

    proc risk noseqcheck; 

    environment open =RD_PROJ.&ENV_NAME;

    setoptions keeplib;
    
    /* trace methods = PVM_ROLLUP;*/
    /*trace flow = all LEVEL= DETAILED ;*/
    runproject Rollup_&PROJECT_SET 
               options = (outall noallscore nomddbs)
               date    = &RUNDATE
               ;
    
    
    environment save noupdatemetadata;
    
    run;

    /*- post-processing -*/
    %post_capital_run;
    %LET cname=%SYSFUNC(CAT(Summary,_,Rollup_&PROJECT_SET));

    /*** END : optimal allocation done and simply aggregate result ***/

    %EXIT:

    /* Create information map for cube if present */
    %if %upcase(&cube_output) eq YES %then %do;

  %get_regrpt_nls_strings; 

  
   

  %let mapname =  &olap_report &for &project_set : &cname;

   %if    %length(&mapname)<60 %then 
          %let namelength=%length(&mapname);
  %else   %let namelength=60;

        
        %infomap (cubename=&cname,mapname=%substr(&mapname,1,&namelength),projecttype=cap_rollup);   
    %end;   
    
    /* check for risk run success */
    %if %dsexist(&RESLIB1..summary) = 0 %then %do;
       %let SYSTEM_FATAL_ERROR = Y;
       %print_msg(error, 2, 153);
    %end;

%mend e15_project_capital_crd4;
