/*****************************************************************************
 * Copyright (c) 2012 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: prepare_crm_allocation_crd4.sas
 *
 * PURPOSE: Prepare for crm allocation. 
 *
 *
 *   USAGE: %prepare_crm_allocation_crd4(EXPCRMDS=, EXPDS=, CRMDS=, NODEDS=, ARCDS=, EXPNOCRM=, EXP1CRM1=, METHOD=,
                                    INVALIDDS= );
 *          EXPCRMDS: linked exposure-crm data set
 *          EXPDS   : exposure data set
 *          CRMDS   : crm data set
 *          NODEDS  : node data set
 *          ARCDS   : arc data set
 *          EXPNOCRM: exposures without crm
 *          EXP1CRM1: unique exposure-crm links
 *          METHOD  : crm allocation method (OPTIMAL / BY_RANK)
 *          INVALIDDS:invalid exposure-crm link
 *   NOTES:  
 *          
 *   UPDATES:
 *    1) 12Aug2008 : STD_SOV_GUARANTEE_PREF_TREATMENT option implemented
 *    2) 12Aug2008 : Corrected implementation of Para-185.
 *    3)  8May2013 : update special cases of CRM RW under financial simple method   
 *    4) 15May2013 : update EAD and CCF for obsi with CRMs 
 *****************************************************************************/

%macro prepare_crm_allocation_crd4(EXPCRMDS=, EXPDS=, CRMDS=, NODEDS=, ARCDS=, EXPNOCRM=, EXP1CRM1=, METHOD=,
                              INVALIDDS=&RESLIB1..all_exposure_crm_invalid);
    
    %local COSTVAR N_EXP_CRM_INVALID N_EXP_NO_RW;
   
/*- remove all exposures without risk weight -*/
    data &EXPDS (index=(EXP_ID))
         &EXPDS._no_rw;
    set &EXPDS;
   %if &RETAIL_EXP_IND_LOAD_USFINAL = YES %then %do;  
            if missing(RW_EXP) then output &EXPDS._no_rw;
     %end;
     %else %do;
            if missing(OUTV_RW_SEG) and COUNTERPARTY_CLASS = "RETAIL" then output &EXPDS._no_rw;
     %end;
    else output &EXPDS;
    run;
    
    %let N_EXP_NO_RW = %attrn(&EXPDS._no_rw, NOBS);
    %if &N_EXP_NO_RW GT 0 %then %do;
      %put ;
      %print_msg(WARNING, 2, 234);
      %put ;
    %end;

    %if &METHOD = OPTIMAL %then %do;
        %let COSTVAR = _COST_;
        %let COSTVAR_DBL_DEF_EFFECT = _COST_DBL_DEF_EFFECT_ ;
    %end;
    %else %do;
        %let COSTVAR = _WEIGHT_;
        %let COSTVAR_DBL_DEF_EFFECT = _WEIGHT_DBL_DEF_EFFECT_ ; 
    %end;
    
    data &EXPCRMDS(index=(EXP_CRM=(EXP_ID CRM_ID) EXP_ID=(EXP_ID)) drop=REASON EST_HOLDING_DAYS 
                   _wgt_ _adj_ R B temp_m OUTV_RW_DILUTION)
         invalid
         (keep=EXP_ID CRM_ID APPROACH ELIGIBLE_CRM_TYPE INCEPTION_DATE 
               CRM_INCEPTION_DATE MATURITY_DATE CRM_MATURITY_DATE REASON RW_EXP &COSTVAR PD_EXP 
               PD_CRM_COUNTERPARTY LINK_MATURITY_DATE rename=(&COSTVAR=RW_CRM)
         );
    set &EXPCRMDS;

    %if &METHOD = OPTIMAL %then %do;
        length _ARCID_ $80.;
    %end;
    
/*- 0: make sure there is a pair -*/
    if missing(EXP_ID) or missing(CRM_ID) then do;
       REASON = "%print_msg(,2,300,1)";
       output invalid;
       goto DONE;
    end;

/*- 1: set FX haircut -*/
/*- apply haircut fx according to the currency pair and scale it if necessary -*/            
    if CCY NE CRM_CCY then do;     
       HAIRCUT_FX = input(put(compress(CCY||":"||CRM_CCY), MAP_HAIRCUT_FX.), best12.);
       if not missing(HAIRCUT_SCALE) then do;
          HAIRCUT_FX = HAIRCUT_FX * HAIRCUT_SCALE;
          EST_HOLDING_DAYS = 
               input(put(compress(CCY||":"||CRM_CCY), MAP_H_FX_HOLDING_DAYS.), best12.);
          if (not missing(HAIRCUT_HOLDING_DAYS)) and (not missing(EST_HOLDING_DAYS)) then 
             HAIRCUT_FX = HAIRCUT_FX * sqrt ( HAIRCUT_HOLDING_DAYS / EST_HOLDING_DAYS );
       end;
    end;
    else do;
       HAIRCUT_FX = 0;
/*- further: carve-out treatment for repo-style transactions: Basel II 170:  -*/
/*-   apply a 0% volatility adjustment under the Financial Collateral Comprehensive method: CRR Article 227  -*/
       if ELIGIBLE_REPO_CARVE_OUT_FLG = 1 and 
    ( (STD_PRODUCT_CLASS = 'REPO_STYLE' and APPROACH='STD') or 
            (IRB_PRODUCT_CLASS in ('REPO_STYLE', 'EML') and APPROACH=: 'IRB')) then do;
          HAIRCUT_EXP = 0;
          HAIRCUT_CRM =0;
       end;
    end;

/*- 2: get CRM_VALUE_DISCOUNT_FLG and LGD_CRM -*/
    CRM_VALUE_DISCOUNT_FLG = 0;

    %if &METHOD = BY_RANK %then %do;
        MIT_HAIRCUT_ADJ = sum(1, HAIRCUT_EXP/100);
    %end;



    if SENIORITY_FLG = 1 then LGD_CRM = LGD_CRM_SEN;
    else LGD_CRM = LGD_CRM_SUB;

/*- 3: find the cost in crm allocation through NETFLOW -*/
    &COSTVAR = .;
    
/*-  adjust CCF to 100% and update EAD value in &EXPCRMDS for OBSIs that have CRMs: Article 222(3),223(4),230(1),235(1) and 236(3) of CRR -*/
    if CCF ne . and EAD_AMT = . and CRM_VALUE >0 and not (CRM_STD_PRODUCT_CLASS = 'IMM_PROPERTY' and CRM_TYPE ='IMM_PROPERTY') then do ; 
       EAD = EXP_VALUE ;
       /*- purchased receivables: */
       if RECEIVABLE_EXPOSURE_FLG = 1 then do ;
          if APPROACH in ('IRB', 'IRBF', 'IRBA') then EAD = FACE_VALUE_AMT * sum(1, - OUTV_RW_DILUTION * &CAPITAL_RATIO / 100); 
          else if APPROACH = 'STD' then EAD = FACE_VALUE_AMT ;
       end ;
       
       CCF =100;
    end ;   
     

/*- financial collateral -*/
    if CRM_TYPE = 'FINANCIAL_COLLATERAL' then 
     do;
     /*- initialize CRM_COMP_FLG = 0 for all financial collaterals. -*/
       CRM_COMP_FLG = 0;
    
     /*-  update all special cases of CRM RWs for financial simple method: CRR Article 222 (3)-(7)  -*/
     if APPROACH = 'STD' and "&STD_CRM_APPROACH" = "SIMPLE" and ELIGIBLE_CRM_TYPE ='FINANCIAL_SIMPLE' then
     do;
      
     /*-  CRR Article 222(6b)&(7): CRM is debt issued by central gov/bank in the same currency as the exposure -*/         
         if  (RW_CRM = 0 and (CRM_COUNTERPARTY_SUBCLASS in ('CENTRAL_GOV','CENTRAL_BANK','INT_ORG','HR_MDB') or CRM_CENTGOV_TRT_FLG eq 1) and CCY = CRM_CCY) then
         do;
             CRM_VALUE_DISCOUNT_FLG = 1;   
             &COSTVAR = 0;
         end;
           
     /*-  CRR Article 222(6a)&(5): CRM is cash in the same currency as the exposure -*/ 
     /*-  The 0% risk weight also applies to gold as a CRM                              -*/ 
         else if (CRM_SUBTYPE EQ 'CASH_INSTITUTION' and CCY = CRM_CCY) or (CRM_SUBTYPE EQ 'GOLD') then
               &COSTVAR = 0;  
               
     /*-  CRR Article 222(3): RW floor for CRMs  -*/    
         else
               &COSTVAR = max(&STD_CRM_SIMPLE_RW_FLOOR, RW_CRM);
               
      /*-  CRR Article 222(4): RW for the cash leg of repo style transactions  -*/
          if STD_PRODUCT_CLASS = 'REPO_STYLE' and ELIGIBLE_REPO_CARVE_OUT_FLG = 1 then do ;
             &COSTVAR = 0 ;  
             if CORE_MARKET_PARTICIPANT_FLG = 0 then &COSTVAR = 10;
          end;
          
     /*- CRR Article 222(5)&(7): RW for debt collaterals of derivative exposures  -*/
         if STD_PRODUCT_CLASS = 'DERIVATIVE' and (CRM_COUNTERPARTY_SUBCLASS in ('CENTRAL_GOV','CENTRAL_BANK','INT_ORG','HR_MDB') or CRM_CENTGOV_TRT_FLG eq 1)
         and RW_CRM = 0 then
             &COSTVAR = 10;
     end;
    /*****************  Updates end here for financial collateral simple method.  **********/
    
     else if APPROACH = 'BASEL_I' then 
            &COSTVAR = max(&STD_CRM_SIMPLE_RW_FLOOR, RW_CRM);
     /**** new added ***/
     else do;
     /*- for financial collaterals using financial collateral comprehensive method -*/
          &COSTVAR = 0;
          CRM_COMP_FLG =1;
/*************************************************************************/
     end;
    end ; 

/*- guarantee, credit derivative and insurance -*/
  else if CRM_TYPE in ('GUARANTEE', 'CREDIT_DERIVATIVE', 'INSURANCE') then 
    do;

/*****************************************************************************************/
        /*  
    if APPROACH = 'STD' or APPROACH='BASEL_I' then do;

            &COSTVAR = RW_CRM_COUNTERPARTY;
    
        end;
    */
/*****************************************************************************************/
    if APPROACH = 'STD' or APPROACH='BASEL_I' then do ;   
       if CRM_SUBTYPE = 'FUNDED' then &COSTVAR = RW_CRM ;
       else if CRM_SUBTYPE ne 'FUNDED' then &COSTVAR = RW_CRM_COUNTERPARTY;       
    
    /*- Article 121(4): adjust RW of trade finance exposure to unrated institution  -*/
    if STD_PRODUCT_CLASS = 'TRADE_FIN' and CRM_COUNTERPARTY_CLASS = 'INSTITUTION' and CRM_EXT_MAP_RATING = 'MISS' then do;             
            &COSTVAR = RW_CRM;
    end;

 
    end ;
    
    if APPROACH = 'STD'  and CRM_COUNTERPARTY_CLASS = 'SOVEREIGN'   and CRM_COUNTERPARTY_SUBCLASS in('CENTRAL_GOV','CENTRAL_BANK')then 
       do;
        if "&STD_SOV_GUARANTEE_PREF_TREATMENT"="YES" and "&STD_SOV_DOM_PREF_TREATMENT" ="YES" then  
           do; 
              if %str(put(COUNTERPARTY_COUNTRY,$MAP_SOV_GUARANTEE_PREF_TREAT.))=%str('YES') then 
                do;             
                   if (CCY = CRM_CCY) and (index(trim(put(COUNTERPARTY_COUNTRY,$MAP_COUNTRY_CURRENCY.)), trim(CCY))) > 0 then
                      &COSTVAR=0; 
                end;
           end; 
     end;
       
     

/*****************************************************************************************/
        
        /*- not sure about SECURITIZATION and EQUITY -*/
     else if APPROACH =: 'IRB' or APPROACH='US_IRB' then 
           do;
          /*- defaulted CRM -*/
             if PD_CRM_COUNTERPARTY = 100 then 
               do;
                if PD_EXP < 100 then 
                  do;
                    REASON = "%print_msg(,2,303,1)";
                    output invalid;
                    goto DONE;
                  end;
                else 
                  do;
                  if LGD_CRM_ADJUSTED_FLG ne 1 then do; /* to check LGD_CRM_ADJUSTED_FLG with 1 should not get updated in COSTVAR */   
                    if "&IRB_USE_GUARANTEE_LGD" = "YES" and LGD_CRM NE . and LGD_EXP NE 0 then 
                      do;
                  /*- use LGD_CRM to adjust RW using the EL -*/
                  /*- recall RW_EXP is already adjusted for EL -*/
                        &COSTVAR = max(0, RW_EXP+(LGD_CRM-LGD_EXP)/&CAPITAL_RATIO*IRB_SCALING_FACTOR);
                      end;
                  /* otherwise no effect of CRM should be accounted and thus &COSTVAR=. */
                    end; 
                 end;
               end;
        /*- double default treatment: meet all the requirements and PD_EXP < 100 -*/   
        /*- in the case of PD_EXP = 100, double default treatment can only increase the RW */ 
            else if ELIGIBLE_DD_TREAT_CRM='Y' and ELIGIBLE_DD_TREAT_GUARANTOR='Y' and 
               APPROACH in ('IRB', 'IRBF', 'IRBA', 'US_IRB') and PD_EXP < 100 and 
               COUNTERPARTY_GROUP NE CRM_COUNTERPARTY_GROUP then do; 
        /*- CRM effective maturity should be used but given the type of CRM, the 
            effective maturity is the same as the residual maturity for now       -*/ 
        /*- no exemption from the floor on the effective maturity -*/
                  if APPROACH = 'IRBF' then temp_m = &IRBF_M;
                  else temp_m= min(&IRBA_M_CAP, max(LINK_RESIDUAL_MATURITY, &IRBA_M_FLOOR));
/*                  if APPROACH = 'US_IRB' then do; */
                  
                    /*-- from US Final Rules page 69418 (4), if the guarantee has immediate payout option, 
                         choose the minimum of lgd exposure and lgd of guarantee.  Ohterwise, use the 
                         lgd of the guarantee --*/
/*                    if CRM_SUBTYPE eq "IMMEDIATE_PAYOUT" then                    */
/*                       LGD_DD = min(LGD_CRM, LGD_EXP);                                              */
/*                    else                       */
/*                       LGD_DD = LGD_CRM; */
/*                    */
/*                    %irb_rw_double_default_usfinal(PD_EXP, PD_CRM_COUNTERPARTY, LGD_DD, temp_m, IRB_SCALING_FACTOR, &COSTVAR); */
                       
                    /* set flag for double default treatment */
/*                    &COSTVAR_DBL_DEF_EFFECT = 1;              */
/*                  end;*/
/*                  else do;*/


                    %irb_rw_double_default(PD_EXP, PD_CRM_COUNTERPARTY, LGD_CRM, temp_m, CRM_FIRM_SIZE, 
                          &IRB_SME_MIN_FIRM_SIZE, &IRB_SME_MAX_FIRM_SIZE, IRB_SCALING_FACTOR, &COSTVAR); 
/*                  end; */
            end;
            
            /* The rw formula below assume pd_crm and lgd_exp. */              
            else do ;
             
              select (CRM_COUNTERPARTY_CLASS);
        
                when ('INSTITUTION') do;
                    %irb_rw_bank_crd4(PD_CRM_COUNTERPARTY, LGD_EXP, MAT_EFF, IRB_SCALING_FACTOR, &COSTVAR,LARGE_FI_ENTITY_FLG);
                end;
            
                when ('CORPORATE') do;
                       %irb_rw_corporate(PD_CRM_COUNTERPARTY, LGD_EXP, MAT_EFF, CRM_FIRM_SIZE, &IRB_SME_MIN_FIRM_SIZE, 
                                      &IRB_SME_MAX_FIRM_SIZE, IRB_SCALING_FACTOR, &COSTVAR); 

                end;
            
                when ('SOVEREIGN') do;
                    if PD_CRM_COUNTERPARTY = 0 then 
                        &COSTVAR = 0;
                    else do;
                        %irb_rw_sovereign(PD_CRM_COUNTERPARTY, LGD_EXP, MAT_EFF, IRB_SCALING_FACTOR, &COSTVAR);
                    end;
                end;

                when ('RETAIL') do;
                    if PD_CRM_COUNTERPARTY = 0 then 
                        &COSTVAR = 0;
                    else do;
                            %irb_rw_retail_ore(PD_CRM_COUNTERPARTY, LGD_EXP, IRB_SCALING_FACTOR, &COSTVAR); 

                    end;
                end;
                
                otherwise;
              end;
/********************************************************************************
    /* For pd substitute approach */

              if LGD_CRM_ADJUSTED_FLG ne 1 and CRM_SUBTYPE eq "IMMEDIATE_PAYOUT" and LGD_CRM NE . and LGD_EXP NE 0 
              then do ;
                   &COSTVAR = Min(&COSTVAR, &COSTVAR * LGD_CRM / LGD_EXP);
                   if LGD_CRM < LGD_EXP then &IRB_USE_GUARANTEE_LGD = YES ; 
                   else if LGD_CRM >= LGD_EXP then &IRB_USE_GUARANTEE_LGD = NO ; 
              end ;
              
              else if LGD_CRM_ADJUSTED_FLG ne 1 and CRM_SUBTYPE ne "IMMEDIATE_PAYOUT" and LGD_CRM NE . and LGD_EXP NE 0
              then do ;
                   &COSTVAR = &COSTVAR * LGD_CRM / LGD_EXP;
                   &IRB_USE_GUARANTEE_LGD = YES ;
              end ;     
                
    /* For lgd_adjustment approach */
              else if LGD_CRM_ADJUSTED_FLG eq 1 and LGD_CRM NE . and LGD_EXP NE 0 then do ;
                   &COSTVAR = max(&COSTVAR* LGD_CRM / LGD_EXP, RW_EXP ) ;     
                   if LGD_CRM < LGD_EXP then &IRB_USE_GUARANTEE_LGD = YES ; 
                   else if LGD_CRM >= LGD_EXP then &IRB_USE_GUARANTEE_LGD = NO ;
              end;
              
            end ; 
          end ;
     end;
/*- STD_physical_collateral -*/ 
    else if CRM_TYPE = 'STD_PHYSICAL_COLLATERAL' then do;     
        &COSTVAR = RW_CRM ;
    end ;    
     
/*- IRB collateral -*/
    else if CRM_TYPE = 'IRB_COLLATERAL' then do; 
       
/*- If LGD under IRBA is already adjusted for crm, it should not be here -*/
/*- Since it is here, treat it as under IRBF: Basel II - 297-298 -*/
/*- check if LGD_EXP is 0.  If it is the risk weight for the exposure will be 0 -*/
/*- No crm is needed anyway -*/
        if LGD_EXP NE 0 then
          &COSTVAR = RW_EXP * LGD_CRM / LGD_EXP;
        
    end;

/*- netting -*/
    else if CRM_TYPE = 'NETTING' then do;

        &COSTVAR = 0;

    end;

/*- bounded below by 0 -*/
    if not missing(&COSTVAR) then &COSTVAR=max(0, &COSTVAR);
    if not missing(&COSTVAR_DBL_DEF_EFFECT) then &COSTVAR_DBL_DEF_EFFECT=max(0, &COSTVAR_DBL_DEF_EFFECT);
    
/*- if "cost" is higher than RW_EXP then the CRM is in effective -*/

   if APPROACH="US_IRB" and SEGMENT_ID NOT IN ('','_#_') then do ;  
            if &COSTVAR >= OUTV_RW_SEG then do;
               REASON = "%print_msg(,2,304,1)";
               output invalid;   
               goto DONE;
            end; 
   end; 
   else if APPROACH="US_IRB" and CRM_TYPE not in ('GUARANTEE','CREDIT_DERIVATIVE','INSURANCE') and &COSTVAR >= RW_EXP then do;
          REASON = "%print_msg(,2,304,1)";
          output invalid;   
          goto DONE;
  end; 
  else if &COSTVAR > RW_EXP then do;
       REASON = "%print_msg(,2,304,1)";
       output invalid;   
       goto DONE;
  end; 

   
    
/*- 4: maturity mismatch -*/
/*- exception: repo-style transactions -*/
    if (STD_PRODUCT_CLASS = 'REPO_STYLE' and APPROACH='STD') or
       (IRB_PRODUCT_CLASS in ('REPO_STYLE', 'EML') and APPROACH not in ('BASEL_I', 'STD')) then 
       MAT_MISMATCH_WEIGHT_CRM = 1;
    else do;
       RESMAT_EXP = min(&MAT_MISMATCH_RESMAT_MAX_EXP, RESIDUAL_MATURITY);
       RESMAT_CRM = min(RESMAT_EXP, LINK_RESIDUAL_MATURITY);
       
       if RESMAT_EXP eq &MAT_MISMATCH_RESMAT_ADJUST and RESMAT_CRM eq RESMAT_EXP then MAT_MISMATCH_WEIGHT_CRM =1; 
       else MAT_MISMATCH_WEIGHT_CRM = (RESMAT_CRM - &MAT_MISMATCH_RESMAT_ADJUST) / (RESMAT_EXP - &MAT_MISMATCH_RESMAT_ADJUST);
       
       /*- maturity mismatch weights for synthetic securitizations of originators: CRR Articles 249-250 -*/
       if SECURITIZATION_EXPOSURE_FLG =1 and SEC_STRUCTURE_TYPE ='SYNTHETIC' and PRIMARY_BANK_ROLE = 'ORIGINATOR' then do ;
          if RW_EXP ne &RW_UNRATED_SECURITIZATION then do ;          
            if LONGEST_MATURITY_DATE ne . then RESMAT_EXP = max(0, YRDIF(&RUNDATE, LONGEST_MATURITY_DATE, "&DAY_BASIS"));
            else if LONGEST_MATURITY_DATE = . then RESMAT_EXP = max(0, YRDIF(&RUNDATE, &EXP_RESIDUAL_MATURITY, "&DAY_BASIS"));
            RESMAT_EXP = min(RESMAT_EXP, 5) ;
            
            RESMAT_CRM = min(RESMAT_EXP, LINK_RESIDUAL_MATURITY);
      
            MAT_MISMATCH_WEIGHT_SEC = (RESMAT_CRM - &MAT_MISMATCH_RESMAT_ADJUST) / (RESMAT_EXP - &MAT_MISMATCH_RESMAT_ADJUST);
            MAT_MISMATCH_WEIGHT_SEC_UNDERLY = (RESMAT_EXP - RESMAT_CRM) / (RESMAT_EXP - &MAT_MISMATCH_RESMAT_ADJUST);
          end;
           
          else if  RW_EXP = &RW_UNRATED_SECURITIZATION then do ;
            MAT_MISMATCH_WEIGHT_SEC = 1;
            MAT_MISMATCH_WEIGHT_SEC_UNDERLY = 0 ;
          end ;
           
          MAT_MISMATCH_WEIGHT_CRM =1;
          
       end ;
       
    end;
    
/*- 5: create flow multiplier -*/
    %if &METHOD = OPTIMAL %then %do;
        _ARCID_ = compress(EXP_ID || CRM_ID);
    %end;

    _MULT_ = MAT_MISMATCH_WEIGHT_CRM;
    %if "&APPLY_CCF_AFTER_CRM" = "YES" %then %do;
    if OFF_BALANCE_SHEET_FLG=1 then 
       _MULT_ = _MULT_ * CCF/100;
    %end;

/*- adjust multiplier for value adjustment and haircut -*/
    if CRM_TYPE = 'FINANCIAL_COLLATERAL' then do;
        if APPROACH = 'STD' and "&STD_CRM_APPROACH" = "SIMPLE" and ELIGIBLE_CRM_TYPE = 'FINANCIAL_SIMPLE' then do;
          /*- CRM_COMP_FLG is defined earlier in this code file. -*/   
         /*   %if &METHOD = BY_RANK %then %do;
                CRM_COMP_FLG = 0;            
            %end;
         */
            if CRM_VALUE_DISCOUNT_FLG = 1 then _MULT_ = _MULT_ / (1 + &STD_CRM_SIMPLE_CRM_DISCOUNT / 100);
            
       end;
       
       else do;
            _MULT_ = _MULT_ * sum(1, -HAIRCUT_CRM / 100, -HAIRCUT_FX / 100) / sum(1, HAIRCUT_EXP / 100);

            %if &METHOD = OPTIMAL %then %do;
                _ARCID_ = compress(_ARCID_ || '***H_E');       
                /* adjust &COSTVAR for the exposure haircut. When &COSTVAR=0 this is not necessary */
                &COSTVAR  = &COSTVAR * sum(1, HAIRCUT_EXP / 100); 
            %end;
           /* %else %do;
                CRM_COMP_FLG = 1;
            %end;
           */ 
            /*- Adjusted exposure in this situation may be greater than the unadjusted exposure -*/
            /*- which should not be considerd by effective mitigation                           -*/
            if EAD*HAIRCUT_EXP / 100 - sum(1, HAIRCUT_EXP / 100) * CRM_VALUE * _MULT_ > 0 then do;
               REASON = "%print_msg(,2,229,1)";
               output invalid;
               goto DONE;
            end;
       end;
    end;

/*- adjust multiplier for currency mismatch -*/
    else if CRM_TYPE in ('GUARANTEE', 'CREDIT_DERIVATIVE', 'INSURANCE', 'NETTING') then do;
        _MULT_ = _MULT_ * sum(1, -HAIRCUT_FX / 100);
    end;

/*- adjust multiplier for over-collateralization for IRB collateral -*/
    else if CRM_TYPE = 'IRB_COLLATERAL' then do;
        _MULT_ = _MULT_ * sum(1, -HAIRCUT_FX / 100) / (C_STAR_STAR / 100);
    end;

    %if &METHOD = OPTIMAL %then %do;    
        /*- adjust &COSTVAR to reflect the flow adjustment _MULT_ -*/
        &COSTVAR = &COSTVAR * _MULT_;
    %end;

/*- check for missing &COSTVAR or _MULT_ -*/
    if missing(&COSTVAR) or missing(_MULT_) then do;
       REASON = "%print_msg(,2,228,1)";
       output invalid; 
       goto DONE;
    end;
      
    output &EXPCRMDS;
    
    DONE : 
    run;
    
/*- update EAD and CCF for &EXPDS table -*/
    proc sort data = &EXPCRMDS(keep=EXP_ID EAD CCF MAT_MISMATCH_WEIGHT_SEC MAT_MISMATCH_WEIGHT_SEC_UNDERLY CRM_COMP_FLG)
                out= "&REGPATH&PATHSEP.exposure_crm_uniq_exp" nodupkey ;                   
       by EXP_ID;
    run ;
    
    data &EXPDS ;
       merge &EXPDS(drop=OUTV_RW_DILUTION) 
             "&REGPATH&PATHSEP.exposure_crm_uniq_exp"(keep=EXP_ID EAD CCF);  
       by EXP_ID ;
    run ;   
         
/*- append to &RESLIB1..all_exposure_crm_invalid -*/
    %let N_EXP_CRM_INVALID = %attrn(&INVALIDDS, NOBS);
    %if &N_EXP_CRM_INVALID GT 0 %then %do;
      proc append base=&INVALIDDS data=invalid force;
      run;
      
      %put ;
      %print_msg(WARNING, 2, 230);
      %put ;
    %end;
    
  
   
/*- 7: prepare data for PROC INTPOINT -*/    
    %if &METHOD = OPTIMAL %then %do;   
      %let N_EXP_CRM = %attrn(&EXPCRMDS, NOBS);
        
      %if &N_EXP_CRM > 0 %then %do;    
        /*- 6: partition the network -*/
        proc compile;
             call partition_network("&EXPCRMDS",'EXP_ID','CRM_ID','networkds', '_NODE_');
        run; 
        
        proc sort data=networkds;
        by _partition_;
        run;
    
        data buckets (index=(_partition_) keep=_partition_ count);
        set networkds;
        by _partition_;
        if first._partition_ then count=0;
        count + 1;
        if last._partition_ then output;
        run;
        
        data networkds (index=(_NODE_));
        merge networkds (in=a) buckets;
        by _partition_;
        if a;
        run;
        
        /*- 7: special cases -*/ 
        /*- exposures without crm -*/
        data &EXPNOCRM (drop = _partition_ count);
        merge &EXPDS
              networkds (in=link rename=(_NODE_=EXP_ID));
        by EXP_ID;    
        if not link;
        run;   
        
        /*- update links -*/
        data &EXP1CRM1 (drop=_partition_ count)
             multiple_links (keep=_ARCID_ EXP_ID CRM_ID _MULT_ _COST_ _COST_DBL_DEF_EFFECT_ RW_EXP _partition_ count index=(_partition_));
        merge &EXPCRMDS (in = exp)
              networkds (in=link rename=_NODE_=EXP_ID);
        by EXP_ID;
        if exp and link;
        /*- single links -*/
        if count=2 then do;
           output &EXP1CRM1 ;
        end;
        /*- multiple links -*/
        else do;
           output multiple_links;
        end;
        run;
      
        data dummy_node;
        length _TYPE_ $3 _NODE_ $32 _SUPDEM_ 8;
        _TYPE_   = 'CRM';
        _NODE_   = 'NOT_COVERED';
        _SUPDEM_ =  0;
        run;
       
        /*- 8: prepare for optimal allocation - multiple links only -*/
        
        /*- partition the network -*/        
        %local N_LINKS SIZE IDATA;
        %let N_LINKS = %attrn(multiple_links, NOBS);
        %let N_NETWORK = %eval(%sysfunc(ceil(&N_LINKS/&NETWORK_SIZE)));
        %if &N_NETWORK = 0 %then %do;
           %goto EXIT;
        %end;
        
        %let SIZE = %eval(%sysfunc(ceil(&N_LINKS/&N_NETWORK)));
        
        %if &N_NETWORK = 1 %then %do;
           data &ARCDS.0(keep=_ARCID_ EXP_ID CRM_ID _MULT_ _COST_ _COST_DBL_DEF_EFFECT_ RW_EXP index=(EXP_ID));
           set multiple_links;
           run;
           
           data &ARCDS.0 (drop=RW_EXP);
           set &ARCDS.0;
           by EXP_ID;
           output;
           if first.EXP_ID then do;
              CRM_ID = 'NOT_COVERED';
              _ARCID_= compress(EXP_ID || 'NOT_COVERED');
              _MULT_ = 1;
              _COST_ = RW_EXP;
              output;
           end; 
           run;
           
           data &NODEDS.0(keep=_TYPE_ _NODE_ _SUPDEM_);
           merge 
                /*- demand nodes -*/
                 &EXPDS (in=exp keep = EXP_ID APPROACH EAD RW_EXP SPECIFIC_PROVISION 
                         %if "&APPLY_CCF_AFTER_CRM" = "YES" %then %do;
                         OFF_BALANCE_SHEET_FLG CCF
                         %end;                                  
                         rename = (EXP_ID = _NODE_ EAD = _SUPDEM_))
                /*- supply (CRM) node -*/
                 &CRMDS (in=crm keep = CRM_ID CRM_VALUE rename = (CRM_ID = _NODE_ CRM_VALUE = _SUPDEM_))
                 
                 networkds (in = link where=(count>2));
                ;
           by _NODE_;
           if link;
           
           if exp then do;
              _TYPE_ = 'EXP';
              /*-  Basel II - 52: Under STD, exposures should be risk-weighted net of specific provision -*/
              if APPROACH = 'STD' then do;
                 %if "&APPLY_CCF_AFTER_CRM" = "YES" %then %do;
                   if OFF_BALANCE_SHEET_FLG=1 then 
                      _SUPDEM_ = - max(0, sum(_SUPDEM_, -SPECIFIC_PROVISION*CCF/100));
                   else
                      _SUPDEM_ = - max(0, sum(_SUPDEM_, -SPECIFIC_PROVISION));
                 %end;
                 %else %do;
                   _SUPDEM_ = - max(0, sum(_SUPDEM_, -SPECIFIC_PROVISION));
                 %end;
               end;
               else _SUPDEM_ = - _SUPDEM_;
           end;
           else do;
               _TYPE_ = 'CRM';
           end; 
           run;

           proc append base=&NODEDS.0 data=dummy_node; run;
        %end; /* N_NETWORK = 1 */
        /*- partition network -*/
        %else %do;      
           data pairs (keep=_partition_ paircount) ;
           set multiple_links;
           by _partition_;
           if first._partition_ then paircount=0;
           paircount + 1;
           if last._partition_ then output;
           run;
           
           /*- assign subnetwork -*/
           data pairs (keep=_partition_ i_subnet index=(_partition_));
           set pairs;
           retain count 0 i_subnet 0;
           count + paircount;
           output;
           if count > &SIZE then do;
              count = 0;
              i_subnet + 1;
           end;
           run;
           
           data networkds (index=(_NODE_));
           merge networkds (where=(count>2)) pairs (in=pair);
           by _partition_;
           run;
           
           
           data %do i=0 %to &N_NETWORK-1; &NODEDS.&i (keep=_TYPE_ _NODE_ _SUPDEM_) %end;;
           merge 
         /*- demand nodes -*/
         &EXPDS (in=exp keep = EXP_ID APPROACH EAD RW_EXP SPECIFIC_PROVISION 
                 %if "&APPLY_CCF_AFTER_CRM" = "YES" %then %do;
                    OFF_BALANCE_SHEET_FLG CCF
                 %end;                                  
                 rename = (EXP_ID = _NODE_ EAD = _SUPDEM_))
         /*- supply (CRM) node -*/
         &CRMDS (in=crm keep = CRM_ID CRM_VALUE rename = (CRM_ID = _NODE_ CRM_VALUE = _SUPDEM_))                 
         networkds (in = link where=(count>2));
         ;
           by _NODE_;
           if link;
             
           if exp then do;
              _TYPE_ = 'EXP';
              /*-  Under STD, exposures should be risk-weighted net of specific provision -*/
              /*- Equity exposures need to have a further weighting applied if off balance sheet -*/ 
              if APPROACH = 'STD' then do;
                 %if "&APPLY_CCF_AFTER_CRM" = "YES" %then %do;
                      if OFF_BALANCE_SHEET_FLG=1 then 
                        _SUPDEM_ = - max(0, sum(_SUPDEM_, -SPECIFIC_PROVISION*CCF/100));
                      else
                        _SUPDEM_ = - max(0, sum(_SUPDEM_, -SPECIFIC_PROVISION));
                 %end;
                 %else %do;
                    if STD_PRODUCT_CLASS = 'EQUITY' and OFF_BALANCE_SHEET_FLG = 1 then
                       _SUPDEM_ = - max(0, sum(_SUPDEM_, -SPECIFIC_PROVISION)*CCF); 
                    else
                       _SUPDEM_ = - max(0, sum(_SUPDEM_, -SPECIFIC_PROVISION));
                 %end;
              end;
              else _SUPDEM_ = - _SUPDEM_;
           end;
           else do;
              _TYPE_ = 'CRM';
           end; 
           
       %do jj = 0 %to &N_NETWORK-1 ;
                if i_subnet = &jj then output &NODEDS.&jj ;
           %end ;
           run;
                
           data %do i=0 %to &N_NETWORK-1; &ARCDS.&i(keep=_ARCID_ EXP_ID CRM_ID _MULT_ _COST_ _COST_DBL_DEF_EFFECT_ RW_EXP index=(EXP_ID)) %end;;
           merge multiple_links pairs (in=pair);
           by _partition_;

       %do jj = 0 %to &N_NETWORK-1 ;
                if i_subnet = &jj then output &ARCDS.&jj ;
           %end ;
           run;
      
           %do i=0 %to &N_NETWORK-1;
               data &ARCDS.&i (drop=RW_EXP);
               set &ARCDS.&i;
               by EXP_ID;
               output;
         
               if first.EXP_ID then do;
                  CRM_ID = 'NOT_COVERED';
                  _ARCID_= compress(EXP_ID || 'NOT_COVERED');
                  _MULT_ = 1;
                  _COST_ = RW_EXP;
                  output;
               end; 
               run;
               
               proc append base=&NODEDS.&i data=dummy_node; run;
            %end;

         %end; /*- N_NETWORK > 1  -*/
      %end; /*- N_EXP_CRM > 0 -*/
      
      %else %do;
        %let N_NETWORK=0;
      /*- all exposures without crm -*/
        data &EXPNOCRM;
        set &EXPDS;
        run;       
      %end;
    %end; /*- OPTIMAL -*/
%EXIT :
%mend prepare_crm_allocation_crd4;
