/*****************************************************************************
 * Copyright (c) 2012 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: r00_post_mtm_5.sas
 *
 * PURPOSE: Process market-related off-balance-sheet items subject to master 
 *          netting agreements 
 *          
 *          
 *
 *   USAGE: 
 *
 *   NOTES: 1. Transactions subject to the same netting agreement must be subject
 *             to the same regulatory approach, i.e. STD/IRBF/IRBA. 
 *          
 *          2. All netting agreements here are bilateral, otherwise they are not
 *             recognized. 
 *
 *****************************************************************************/



%macro r00_post_mtm_5;

%global N_OBSI_NET ;

/*- 1: get all market related off-balance-sheet items subject to MNAs -*/
/*-    CCR_APPROACH = 'CEM' should cover both PE_APPROACH = 'CE' and PE_APPROACH = 'OE' methods -*/
    data _all_exp_obsi_;
    set &RESLIB1..all_exposures;
    where (PE_APPROACH = 'CE' or CCR_APPROACH = 'CEM' ) and NETTING_SET_RK NE .;
    run;

/*- if no such exposures, exit -*/
    %local N_OBSIS;
    %let N_OBSIS = %attrn(_all_exp_obsi_, NOBS);
    %if &N_OBSIS = 0 %then %do ;
        %let N_OBSI_NET = 0 ;
        %goto EXIT;    
    %end ;
   

/******************************************************************************************************
/*- Remove the following part of code because OTC derivatives with negative exposure values should not  
/*- be treated as on-balance sheet netting items which belong to &RESLIB1..crm_netting table. 
/******************************************************************************************************

/*- get all the netting positions that are related to this treatment -
    proc sort data=_all_exp_obsi_ out=nettings nodupkey;
    by NETTING_SET_RK;
    run;
   
    data _all_obsi_netting 
         (drop = CRM: ELIGIBLE_CRM_: OUTV_CRM: OUTV_GENERAL_PROVISION PROVISION: OUTV_EXPOSURE_PROVISION 
                 NOTE OUTV_HAIRCUT_SCALE ELIGIBLE_DD_TREAT_CRM ELIGIBLE_DD_TREAT_GUARANTOR
                 COLL_LEVEL_BUCKET FIRST_TO_DEFAULT_PROT_FLG LGD_BUCKET_: PROTECTION_VALUE_AMT 
                   UNFUNDED_FLG USED_AMOUNT INSTSOURCE INSTNUMBER 
                 OUTV_CCF_REG_Y OUTV_EAD_REG_Y
          rename = (INSTID = EXP_ID
                    VALUE = EXP_VALUE OUTV_EAD = EAD OUTV_ADD_ON = ADD_ON OUTV_HAIRCUT = HAIRCUT OUTV_CCF = CCF
                    OUTV_MAT_EFF = MAT_EFF OUTV_RESIDUAL_VALUE = RESIDUAL_VALUE CURRENCY=CCY )
                   )
          &RESLIB1..crms_netting (drop= HAIRCUT CCY)
          ;
    merge &RESLIB1..crms_netting (in=a)
          nettings (in=b);
          
    if a and b then
       output _all_obsi_netting;
    else if a then
       output &RESLIB1..crms_netting;
    run;  
    
    proc append base=_all_exp_obsi_ data=_all_obsi_netting force; run;
    
/******************************************************************************************/
   
   proc sort data=_all_exp_obsi_; by NETTING_SET_RK; run;


/*- get notional amount-weighted average maturity -*/
     
        
        proc summary data=_all_exp_obsi_(keep=NETTING_SET_RK FACE_VALUE_AMT INCEPTION_DATE MATURITY_DATE MAT_EFF  RESIDUAL_MATURITY);
        var INCEPTION_DATE MATURITY_DATE MAT_EFF RESIDUAL_MATURITY;
        weight FACE_VALUE_AMT;
        by NETTING_SET_RK;
        output out=_all_obsi_mat_(keep=NETTING_SET_RK INCEPTION_DATE MATURITY_DATE MAT_EFF RESIDUAL_MATURITY) 
        mean=INCEPTION_DATE MATURITY_DATE MAT_EFF RESIDUAL_MATURITY;
        run;
        
        data _all_obsi_mat_;
        set _all_obsi_mat_;
        MAT_EFF =  min(&IRBA_M_CAP, max(&IRBA_M_FLOOR, RESIDUAL_MATURITY));
        run;

/*- calculate NGR  -*/
    /*- compute the following -*/
    /*- NCCE: Net Current Credit Exposure and                            -*/
    /*- GCCE: Gross Current Credit Exposure per netting set/counterparty -*/
    /*- PFCE: Potential Future Credit Exposure                           -*/
    /*- NGR: Net-to-Gross Ratio                                          -*/
    data /*- per netting set/counterparty -*/
         _all_obsi_ngr_set_(keep=NETTING_SET_RK SUM_FACE NCCE GCCE PFCE NGR SUM_MKT_RISK_CHG SUM_SETT_RISK_CHG)
         _all_obsi_ngr_all_(keep=SUM_NCCE SUM_GCCE NGR);
    set _all_exp_obsi_(keep=NETTING_SET_RK EXP_VALUE FACE_VALUE_AMT ADD_ON PE_APPROACH MKT_RISK_CHG_AMT SETT_RISK_CHG_AMT) end=lastobs;
    by NETTING_SET_RK;
    retain SUM_FACE NCCE GCCE PFCE SUM_NCCE SUM_GCCE SUM_MKT_RISK_CHG SUM_SETT_RISK_CHG 0;
    
    if PE_APPROACH = 'CE' then do ;
      
       if first.NETTING_SET_RK then do;
           SUM_FACE = 0;
           NCCE = 0;
           GCCE = 0;
           PFCE = 0;
           SUM_MKT_RISK_CHG=0;
           SUM_SETT_RISK_CHG=0;
       end;

       SUM_FACE = sum(SUM_FACE, FACE_VALUE_AMT);
       NCCE = sum(NCCE, EXP_VALUE);
       GCCE = sum(GCCE, max(0, EXP_VALUE));
       PFCE = sum(PFCE, FACE_VALUE_AMT*(ADD_ON/100));
       SUM_MKT_RISK_CHG = sum(SUM_MKT_RISK_CHG, MKT_RISK_CHG_AMT);
       SUM_SETT_RISK_CHG = sum(SUM_SETT_RISK_CHG, SETT_RISK_CHG_AMT);
         
       if last.NETTING_SET_RK then do;
           NCCE = max(0, NCCE);
           if GCCE = 0 then NGR = 1;
           else NGR = NCCE/GCCE;
           
           output _all_obsi_ngr_set_;
           SUM_NCCE = sum(SUM_NCCE, NCCE);
           SUM_GCCE = sum(SUM_GCCE, GCCE);
       end;
    
    end;

    if PE_APPROACH = 'OE' then do ;
    
       if first.NETTING_SET_RK then do;
               SUM_FACE = 0;
               PFCE = 0;
               SUM_MKT_RISK_CHG=0;
               SUM_SETT_RISK_CHG=0;
       end;
       
       SUM_FACE = sum(SUM_FACE, FACE_VALUE_AMT);
       PFCE = sum(PFCE, FACE_VALUE_AMT*(ADD_ON/100));
       SUM_MKT_RISK_CHG = sum(SUM_MKT_RISK_CHG, MKT_RISK_CHG_AMT);
       SUM_SETT_RISK_CHG = sum(SUM_SETT_RISK_CHG, SETT_RISK_CHG_AMT);
       
       if last.NETTING_SET_RK then do;
                  NCCE = 0;
                  GCCE = 0;
                  NGR = 1;
                  output _all_obsi_ngr_set_;
                  SUM_NCCE = sum(SUM_NCCE, NCCE);
                  SUM_GCCE = sum(SUM_GCCE, GCCE);
       end;
       
    end;
         
    if lastobs then do;
        if SUM_GCCE = 0 then NGR =1 ;
        else NGR = SUM_NCCE/SUM_GCCE;     
        output _all_obsi_ngr_all_;
    end;
      
    run;

    %local AGGREGATE_NGR;
    data _null_;
    set _all_obsi_ngr_all_;
    call symput('AGGREGATE_NGR', NGR);
    run;

/*- get all CRMs related to obsi transactions -*/
    proc sql;
    create table _all_crm_obsi_(index=(NETTING_SET_RK)) as
    select exp.NETTING_SET_RK, crm.ISSUE_KEY, crm.HAIRCUT as CRM_HAIRCUT, crm.CRM_VALUE, crm.HAIRCUT_HOLDING_DAYS,
           crm.HAIRCUT_SCALE, crm.CCY as CRM_CCY
    from &RESLIB1..all_crms as crm 
    inner join RD_POS.exposure_crm_link as link 
    on crm.CRM_ID = link.CRM_ID
    inner join _all_exp_obsi_ as exp 
    on exp.EXP_ID = link.EXP_ID;
    quit;

   data _all_obsi_;
    set _all_exp_obsi_(in=exp keep=NETTING_SET_RK ISSUE_KEY HAIRCUT CCY FACE_VALUE_AMT EAD /*EXP_VALUE*/)
        _all_crm_obsi_(in=crm rename=(CRM_CCY=CCY))
        ;
    by NETTING_SET_RK;
    run; 
 
   
   /*- 2: get total exposure, total CRMs, and netted position in each security and currency -*/
  /*- get netting set information -*/
 /*- for OTC derivatives, haircuts apply only to collaterals, not exposures: CRR Article 223(1) and 223(3) -*/
    data _all_obsi_(drop=EST_HOLDING_DAYS 
                    index=(SET_SEC=(NETTING_SET_RK ISSUE_KEY) SET_FX=(NETTING_SET_RK CCY)));
    merge _all_obsi_(in=obsi)
          RD_POS.netting_set
          (in=net_set keep=NETTING_SET_RK NETTING_SETTLEMENT_CURRENCY);
    by NETTING_SET_RK;
    if obsi and net_set;

    if CCY NE NETTING_SETTLEMENT_CURRENCY then do;
        HAIRCUT_FX = input(put(compress(CCY||":"||NETTING_SETTLEMENT_CURRENCY), MAP_HAIRCUT_FX.), best12.);
        if not missing(HAIRCUT_SCALE) then do;
           EST_HOLDING_DAYS = 
             input(put(compress(CCY||":"||NETTING_SETTLEMENT_CURRENCY), MAP_H_FX_HOLDING_DAYS.), best12.);
           if (not missing(HAIRCUT_HOLDING_DAYS)) and (not missing(EST_HOLDING_DAYS)) then  
             HAIRCUT_FX = HAIRCUT_FX * HAIRCUT_SCALE * sqrt ( HAIRCUT_HOLDING_DAYS / EST_HOLDING_DAYS );
        end;
    end;
    else
        HAIRCUT_FX = 0;
    run;

   /*- get net position in each security -*/
    data _all_obsi_sec_(keep=NETTING_SET_RK SUM_FACE SUM_E SUM_C SUM_SEC CRM_TYPE);
    set _all_obsi_(keep=NETTING_SET_RK ISSUE_KEY FACE_VALUE_AMT EAD /*EXP_VALUE*/ CRM_VALUE HAIRCUT CRM_HAIRCUT); 
    by NETTING_SET_RK ISSUE_KEY;
    retain SUM_FACE SUM_E SUM_C SUM_SEC NET_SEC 0;
    length CRM_TYPE $32.;

    if first.NETTING_SET_RK then do;
        SUM_E   = 0;
        SUM_C   = 0;  
        SUM_SEC = 0;
    end;

    /*SUM_E = sum(SUM_E, EAD); */
    SUM_C = sum(SUM_C, CRM_VALUE); 
    
    if first.ISSUE_KEY then do;
        NET_SEC = 0;
    end;

    /*- absolute value of the netted position in a given security -*/
    NET_SEC = sum (NET_SEC, CRM_VALUE*sum(0, CRM_HAIRCUT)/100);
    
    if last.ISSUE_KEY then do;
        NET_SEC = abs(NET_SEC);
        SUM_SEC = sum(SUM_SEC, NET_SEC);
        CRM_TYPE = 'FINANCIAL_COLLATERAL' ;
    end;
       
    if last.NETTING_SET_RK then output;

    run;

   /*- get net position in each currency -*/
    data _all_obsi_fx_(keep=NETTING_SET_RK SUM_FX);
    set _all_obsi_(keep=NETTING_SET_RK CCY EAD /*EXP_VALUE*/ CRM_VALUE HAIRCUT_FX);
    by NETTING_SET_RK CCY;
    retain SUM_FX NET_FX 0;
    
    if first.NETTING_SET_RK then do;
        SUM_FX = 0;
    end;

    if first.CCY then do;
        NET_FX = 0;
    end;

    /*- absolute value of the netted position in a given currency -*/
    NET_FX = sum(NET_FX, CRM_VALUE*sum(0, HAIRCUT_FX)/100);

    if last.CCY then do;
        NET_FX = abs(NET_FX);
        SUM_FX = sum(SUM_FX, NET_FX);
    end;

    if last.NETTING_SET_RK then output;

    run;


/*- 3: create a netted position for each netting set -*/
/*- and inherit other useful information, especially counterparty info -*/  
    data &RESLIB1..all_exposures_obsi_net
         (drop=SUM_: NCCE GCCE NGR PFCE NETTING_SET_INCEPTION_DATE NETTING_SET_MATURITY_DATE 
          index=(NETTING_SET_RK));   
    merge _all_exp_obsi_(in=exp_obsi)
          RD_POS.netting_set
          (keep=NETTING_SET_RK NETTING_SETTLEMENT_CURRENCY VAR_RT INCEPTION_DATE MATURITY_DATE
           rename=(NETTING_SETTLEMENT_CURRENCY=CCY  
                   INCEPTION_DATE = NETTING_SET_INCEPTION_DATE MATURITY_DATE=NETTING_SET_MATURITY_DATE
                  )
           in=net_set
          )
          _all_obsi_mat_(in=wa_mat)
          _all_obsi_sec_(in=pos_sec)
          _all_obsi_fx_(in=pos_fx)
          _all_obsi_ngr_set_(in=ngr_set)
          ;
    by NETTING_SET_RK;

    if exp_obsi and net_set and wa_mat and pos_sec and pos_fx and ngr_set;

    /*- only need info from the first exposure -*/
    if first.NETTING_SET_RK then do;
    
        /*- create EXP_ID for the netted position -*/
        EXP_ID = compress('MNA_OBSI_' || put(NETTING_SET_RK, 16.));    
        
/*- user may want to use netting set maturity info if not missing: insert code here -*/
        if NETTING_SET_MATURITY_DATE ne . and NETTING_SET_INCEPTION_DATE ne . then do;
           %get_maturity(NETTING_SET_INCEPTION_DATE, NETTING_SET_MATURITY_DATE);
        end;   
        else do;
           %get_maturity(INCEPTION_DATE, MATURITY_DATE);
        end;
  
  
        /*- update values -*/
        /*- face value -*/
        FACE_VALUE_AMT = SUM_FACE;
        /*- total exposure value -*/
        if PE_APPROACH = 'CE' then EXP_VALUE = NCCE;
        else if PE_APPROACH = 'OE' then EXP_VALUE = PFCE;
        /*- weighted average add-on (in percentage) -*/
        ADD_ON = (PFCE/FACE_VALUE_AMT)*100;
        MKT_RISK_CHG_AMT = SUM_MKT_RISK_CHG ;
        SETT_RISK_CHG_AMT = SUM_SETT_RISK_CHG ;

        %if %upcase(&NGR_METHOD) = AGGREGATE_BASIS %then %do;
        NGR = &AGGREGATE_NGR;
        %end;

             SUM_E = sum(NCCE, &GROSS_ADD_ON_WEIGHT * PFCE, (1 - &GROSS_ADD_ON_WEIGHT) * NGR * PFCE);
                EAD_REG_Y = sum(NCCE, &GROSS_ADD_ON_WEIGHT * PFCE, (1 - &GROSS_ADD_ON_WEIGHT) * NGR * PFCE);
                       
                /*- VaR approach -*/
                if VAR_RT NE . then do;
                   EAD_ADJ = max(0, sum(SUM_E, -SUM_C)*sum(1, VAR_RT/100));
                end;
                /*- haircut approach -*/
                else do;
                   EAD_ADJ = max(0, sum(SUM_E, -SUM_C, SUM_SEC, SUM_FX));  
                end;
                /*- update the floor of the weighted average maturity -*/
                if APPROACH in ('IRB', 'IRBA','US_IRB') then
                    MAT_EFF = max(&IRBA_M_FLOOR_REPO_NETTING, MAT_EFF);
                    
                EXP_VALUE = NCCE;/* correction code for netting* test***/
                EAD = EAD_ADJ ;/* correction code for netting*test***/
                output;
                
            end;  
            
    run;

    %let N_OBSI_NET = %attrn(&RESLIB1..all_exposures_obsi_net, NOBS);

    %if &N_OBSI_NET > 0 %then %do;
/*- 4: append these netted positions to exposures -*/
/*- original exposures are also kept for later processing -*/
/*- to see the effect of netting -*/    

    data  &reslib1..all_exp_removed_ccr_net_pos;
     length NOTE $100. ;
                set _all_exp_obsi_;
                NOTE="These exposures belongs to a CCR netting agreement and are summed up in a new CCR netting instrument";
    run;

    proc sql;
         update  &RESLIB1..All_exposures set EAD=0 where exp_id in (select exp_id from &RESLIB1..All_exp_removed_ccr_net_pos);
         update  &RESLIB1..All_exposures set EXP_VALUE=0 where exp_id in (select exp_id from &RESLIB1..All_exp_removed_ccr_net_pos);
    quit;



        %if &NETEXPDATA EQ YES  %then %do;
        %if &N_REPO_NET <= 0 %then %do;
            proc sort data=RD_netting_exposures;
          by ORIG_NETTING_SET_RK;
            run;
        %end;
        data &RESLIB1..all_exposures_obsi_net;
        merge &RESLIB1..all_exposures_obsi_net (in=a)
              "&NETEXPPATH.&pathsep.instvals"(drop=ORIG_NETTING_SET_RK VALUE MARGIN_PERIOD_DAYS_CNT 
                                                   MARGIN_AGREEMENT_THRESHOLD_AMT COLLATERAL SHORTPOSITION
                                                   HOLDING FACE_VALUE_AMT MAX_ORIGINAL_MATURITY OUTV_EAD
                                                   OUTV_MAT_EFF OUTV_EXPOSURE_PROVISION OUTV_GENERAL_PROVISION
                                                   OUTV_CRM: OUTV_HAIRCUT_SCALE INSTSOURCE INSTNUMBER INSTTYPE 
                                                   OUTV_ADD_ON CURRENCY BASEDATE MAX_RESIDUAL_MATURITY INSTID
                                                   RESIDUAL_MATURITY ORIGINAL_MATURITY /*FACE_VALUE_AMT*/ MKT_RISK_CHG_AMT SETT_RISK_CHG_AMT 
                                              rename =(OUTV_HAIRCUT=HAIRCUT 
                                                       OUTV_CCF=CCF OUTV_RESIDUAL_VALUE=RESIDUAL_VALUE)
                                              in=b) 
        by NETTING_SET_RK;
        if a and b;
        run;
        %end;
        
        proc append base=&RESLIB1..all_exposures data=&RESLIB1..all_exposures_obsi_net force;run;
    
/*- report -*/
        %put ;
        %print_msg(note,2,48);
        %print_msg(note,2,49);
        %print_msg(note,2,47);
        %put ;
    %end;

    %EXIT:
  
/*******************************************************************************************    
/*- merge the unused netting positions to the crms (moved from post_mtm_0 
    data &RESLIB1..crms_netting;
    set &RESLIB1..crms_netting(drop = APPROACH: EXP: ADDON: CCF: SPECIFIC_PROVISION NOTE OUTV_FACE_VALUE
                   OUTV_MAT_EFF OUTV_EAD OUTV_RESIDUAL_VALUE VALUE OUTV_ADD_ON OUTV_CCF
                   OUTV_GENERAL_PROVISION  OUTV_EXPOSURE_PROVISION INSTSOURCE INSTNUMBER /*PROVISION:*/
                   /*ADD_ON EAD EAD_REG_Y MAT_EFF RESIDUAL_VALUE RECEIVABLE_EXPOSURE_FLG  HEDGE_PAIR_PORTION
                   REMAINING_EQ_EXP EAD_ADJ */ 
   /*         rename = (INSTID = CRM_ID OUTV_CRM = CRM_VALUE OUTV_HAIRCUT = HAIRCUT CURRENCY=CCY 
                      OUTV_HAIRCUT_SCALE=HAIRCUT_SCALE HOLDING_PERIOD_DAYS=HAIRCUT_HOLDING_DAYS));
    run;
    
    proc append base=&RESLIB1..all_crms data=&RESLIB1..crms_netting force;
    run;
************************************************************************************************/
    %clean_lib;      


%mend r00_post_mtm_5;
