/*****************************************************************************
 * Copyright (c) 2012 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: r00_post_mtm_6_crd4.sas
 *
 * PURPOSE: Check for granularity criteria for regulatory retail portfolio. 
 *          
 *   USAGE: 
 *
 *   NOTES: More qualifying criteria can be added here as needed.
 *          
 *
 *****************************************************************************/

%macro r00_post_mtm_6_crd4;

    %if APPROACH = 'BASEL_I' %then %goto EXIT;

/*- qualifying criteria for regulatory retail portfolio -*/
    %local TOTAL_RETAIL_EXPOSURE_STD TOTAL_RETAIL_EXPOSURE_IRB;
    %let TOTAL_RETAIL_EXPOSURE_STD  = 0;
    %let TOTAL_RETAIL_EXPOSURE_IRB  = 0;

/*- 1: get all non-receivable and non-sec-underlying retail exposures -*/
    data _all_retail_(keep=EXP_ID EXP_VALUE COUNTERPARTY_RK APPROACH COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS
                           EXT_ISSUE_MAP_RATING SHORT_TERM_ASSESS_FLG STD_RW_BUCKET EXT_MAP_RATING
                           STD_PRODUCT_CLASS STD_PRODUCT_SUBCLASS IRB_PRODUCT_CLASS IRB_PRODUCT_SUBCLASS
                           PAST_DUE_CLAIM_FLG IN_DEFAULT_FLG 
                      index=(COUNTERPARTY_RK)
                     );
    set &RESLIB1..all_exposures end=last;
    where COUNTERPARTY_CLASS = 'RETAIL' and RECEIVABLE_EXPOSURE_FLG NE 1 and SEC_UNDERLYING_EXP_FLG NE 1
          and ((APPROACH='STD' and STD_PRODUCT_CLASS not in ('SEC', 'SECURITIZATION')) 
          or (APPROACH='IRB' and IRB_PRODUCT_CLASS not in ('SEC', 'SECURITIZATION')));

    retain TOTAL_RETAIL_EXPOSURE_STD TOTAL_RETAIL_EXPOSURE_IRB 0;
    
    /*- total exposure for STD regulatory portfolio -*/    
        if APPROACH = 'STD' and not (STD_PRODUCT_CLASS ='MORTGAGE' and STD_PRODUCT_SUBCLASS ='RESIDENTIAL')
         and not (PAST_DUE_CLAIM_FLG = 1 or IN_DEFAULT_FLG = 1) then 
            /* past dues should be excluded from portfolio size for granularity : Basel2 - 76 */ 
            TOTAL_RETAIL_EXPOSURE_STD = sum(TOTAL_RETAIL_EXPOSURE_STD, max(EXP_VALUE,0));
    /*- total exposure for IRB regulatory portfolio -*/
        else if APPROACH = 'IRB' then
            TOTAL_RETAIL_EXPOSURE_IRB = sum(TOTAL_RETAIL_EXPOSURE_IRB, max(EXP_VALUE,0));

    if last then do;
        /*- create macro variable for total exposure for regulatory retail portfolio -*/
        /*- under STANDARDIZED approach: Basel II - 70 -*/    
        call symput('TOTAL_RETAIL_EXPOSURE_STD', TOTAL_RETAIL_EXPOSURE_STD);
        /*- create macro variable for total exposure for regulatory retail portfolio -*/
        /*- under IRB approach: Basel II - 231 - 234 -*/        
        call symput('TOTAL_RETAIL_EXPOSURE_IRB', TOTAL_RETAIL_EXPOSURE_IRB);
    end;
    run;

/*- 2: get retail portfolio size under STD and IRB approaches and size for each counterparty -*/
    data cpty_retail
    (keep=COUNTERPARTY_RK TOTAL_RETAIL_CPTY_STD TOTAL_RETAIL_CPTY_IRB TOTAL_RETAIL_REVOLV_CPTY_STD TOTAL_RETAIL_REVOLV_CPTY_IRB);
    set _all_retail_(keep=EXP_VALUE COUNTERPARTY_RK APPROACH STD_PRODUCT_CLASS STD_PRODUCT_SUBCLASS IRB_PRODUCT_CLASS
              PAST_DUE_CLAIM_FLG IN_DEFAULT_FLG IRB_PRODUCT_SUBCLASS);
    by COUNTERPARTY_RK;
    
    retain TOTAL_RETAIL_CPTY_STD TOTAL_RETAIL_CPTY_IRB TOTAL_RETAIL_REVOLV_CPTY_STD TOTAL_RETAIL_REVOLV_CPTY_IRB 0;

    if first.COUNTERPARTY_RK then do;
        TOTAL_RETAIL_CPTY_STD = 0;
        TOTAL_RETAIL_CPTY_IRB = 0;
        TOTAL_RETAIL_REVOLV_CPTY_STD = 0;
        TOTAL_RETAIL_REVOLV_CPTY_IRB = 0;
    end;

    if APPROACH = 'STD' then do;
        /*- total retail exposure under STD for every counterparty -*/
        if not (STD_PRODUCT_CLASS = 'MORTGAGE' and STD_PRODUCT_SUBCLASS ='RESIDENTIAL') 
           then TOTAL_RETAIL_CPTY_STD = sum(TOTAL_RETAIL_CPTY_STD, EXP_VALUE);
        /*- total retail revolving exposure under STD for every counterparty -*/ 
        if STD_PRODUCT_CLASS = 'REVOLVING' then
            TOTAL_RETAIL_REVOLV_CPTY_STD = sum(TOTAL_RETAIL_REVOLV_CPTY_STD, EXP_VALUE);
    end;
    
    else do;
        /*- total retail exposure under IRB for every counterparty -*/
        /* residential real estate are excluded from the total: B2 231 and CRD art.86-4(a) */
        if not (IRB_PRODUCT_CLASS = 'MORTGAGE' and IRB_PRODUCT_SUBCLASS = 'RESIDENTIAL') then
        TOTAL_RETAIL_CPTY_IRB = sum(TOTAL_RETAIL_CPTY_IRB, EXP_VALUE);
        /*- total retail revolving exposure under IRB for every counterparty -*/
        if IRB_PRODUCT_CLASS = 'REVOLVING' then
            TOTAL_RETAIL_REVOLV_CPTY_IRB = sum(TOTAL_RETAIL_REVOLV_CPTY_IRB, EXP_VALUE);
    end;

    if last.COUNTERPARTY_RK then output;
    run;
    
/*- 3: retail portfolio qualifying criteria check -*/
    data &RESLIB1..retail_granularity_exception
        (where=(CPTY_CLASS_CHANGED_FLG = 1 or IRB_PRODUCT_CLASS_CHANGED_FLG = 1)
         keep=EXP_ID COUNTERPARTY_RK APPROACH COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS EXT_ISSUE_MAP_RATING STD_RW_BUCKET 
              IRB_PRODUCT_CLASS CPTY_CLASS_CHANGED_FLG IRB_PRODUCT_CLASS_CHANGED_FLG EXT_MAP_RATING
         index=(EXP_ID)
        );    
    merge _all_retail_ cpty_retail;
    by COUNTERPARTY_RK;
    
    CPTY_CLASS_CHANGED_FLG = 0;
    IRB_PRODUCT_CLASS_CHANGED_FLG = 0;
 
/*- qualifying criteria for regulatory retail portfolio under STD approach -*/
%if "&USE_UNDRAWN_RETAIL_GRAN_CHECK" = "YES" %then %do ;
    if FACILITY_FLG ne 1 ;
%end ;

    if APPROACH = 'STD' then do;
        if not (STD_PRODUCT_CLASS = 'MORTGAGE' and STD_PRODUCT_SUBCLASS = 'RESIDENTIAL') then do;
            if /*(TOTAL_RETAIL_CPTY_STD > ((&STD_RETAIL_CPTY_REL_LIMIT / 100) *  (&TOTAL_RETAIL_EXPOSURE_STD + 0))) or*/ 
               (TOTAL_RETAIL_CPTY_STD > &STD_RETAIL_CPTY_ABS_LIMIT) then do;

                COUNTERPARTY_CLASS = 'CORPORATE';
                if COUNTERPARTY_SUBCLASS = 'RETAIL' then COUNTERPARTY_SUBCLASS = 'CORPORATE';
                CPTY_CLASS_CHANGED_FLG = 1;
                /*- only non past dues are assigned a new rw_bucket as bucket for past dues assigned earlier -*/
                    if not (PAST_DUE_CLAIM_FLG = 1 or IN_DEFAULT_FLG = 1) then do; 
                        if SHORT_TERM_ASSESS_FLG = 1 then
                           STD_RW_BUCKET = put(EXT_ISSUE_MAP_RATING, $MAP_STD_RW_BUCKET_SHORT.);
                        else
                           STD_RW_BUCKET = put(EXT_ISSUE_MAP_RATING, $MAP_STD_RW_BUCKET_CORPORATE.);
                end;
            end;
        end;    
     end;
     
   else if APPROACH ne 'STD' then do; 
/*- qualifying criteria for regulatory retail portfolio under IRB approach -*/        
        if COUNTERPARTY_SUBCLASS = 'SME' then do;
            if TOTAL_RETAIL_CPTY_IRB >= &IRB_RETAIL_SME_CPTY_ABS_LIMIT then do;
                COUNTERPARTY_CLASS = 'CORPORATE';
                CPTY_CLASS_CHANGED_FLG = 1;
                /*- change approach if COUNTERPARTY_CLASS is changed to CORPORATE -*/
                APPROACH = 'IRBA';
            end;
        end;
/*- qualifying criteria for regulatory retail revolving portfolio under IRB approach -*/        
        else if IRB_PRODUCT_CLASS = 'REVOLVING' then do;
            if TOTAL_RETAIL_REVOLV_CPTY_IRB > &IRB_RETAIL_QRRE_CPTY_ABS_LIMIT then do;
                IRB_PRODUCT_CLASS = 'OTHER';        
                IRB_PRODUCT_CLASS_CHANGED_FLG = 1;
            end;
        end;
    end;

run;

    proc sort data=rd_pos.counterparty out=&RESLIB1..counterparty;
    by CPTY_RK;
    run;

/*- 4: update for these retail exposures -*/

/*- sort the data set before the merge if netted repo-style or -*/
/*- off-balance sheet exposures have been added -*/

  %if &N_REPO_NET > 0 or &N_OBSI_NET > 0 %then %do ;
        proc sort data = &RESLIB1..all_exposures ;
            by EXP_ID ;
    run ;
    %end 

    %local NCHANGED;
    %let NCHANGED = %attrn(&RESLIB1..retail_granularity_exception, NOBS);
    %if &NCHANGED > 0 %then %do;
    
        data &RESLIB1..all_exposures;
        merge &RESLIB1..all_exposures(in=exp)
              &RESLIB1..retail_granularity_exception(keep=EXP_ID COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS 
                                                          IRB_PRODUCT_CLASS STD_RW_BUCKET APPROACH in=retail)
              ;
        by EXP_ID;
        run;
        
        proc sort data=&RESLIB1..retail_granularity_exception (where=(CPTY_CLASS_CHANGED_FLG=1))
                  out=cpty_retail_1 (keep=COUNTERPARTY_RK EXT_MAP_RATING COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS) nodupkey;
        by COUNTERPARTY_RK;
        run;
        
        data &RESLIB1..counterparty(drop=EXT_MAP_RATING);
        merge &RESLIB1..counterparty (in=a)
              cpty_retail_1(in=b rename=(COUNTERPARTY_RK=CPTY_RK COUNTERPARTY_CLASS=CPTY_CLASS
                                         COUNTERPARTY_SUBCLASS=CPTY_SUBCLASS));
        by CPTY_RK;
        if a;
        
        if b then 
           STD_RW_BUCKET_CPTY = put(EXT_MAP_RATING, $MAP_STD_RW_BUCKET_CORPORATE.);
        run;

/* For checking retail exposure class criteria for unsecured residential mortgages */
/*remove from crd4 */
/*%if  %upcase("&STD_MORTGAGE_FULLY_SECURED") = "NO" %then %do; 

/*- qualifying criteria for regulatory retail portfolio in residential mortgages -
    %local TOTAL_RM_EXPOSURE_STD;
    %let TOTAL_RM_EXPOSURE_STD =  0; 

    data _all_retail_rme(keep=EXP_ID EXP_VALUE COUNTERPARTY_RK APPROACH COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS
                           EXT_ISSUE_MAP_RATING SHORT_TERM_ASSESS_FLG STD_RW_BUCKET EXT_MAP_RATING
                           STD_PRODUCT_CLASS STD_PRODUCT_SUBCLASS IRB_PRODUCT_CLASS IRB_PRODUCT_SUBCLASS
                           PAST_DUE_CLAIM_FLG IN_DEFAULT_FLG
                      index=(COUNTERPARTY_RK)
                     );
    set &RESLIB1..all_exposures end=last;
    where (APPROACH='STD' and STD_PRODUCT_CLASS ='MORTGAGE' and 
            STD_PRODUCT_SUBCLASS ='RESIDENTIAL');

    retain TOTAL_RM_EXPOSURE_STD 0;
    
    if PAST_DUE_CLAIM_FLG NE 1 or IN_DEFAULT_FLG ne 1 then
            /* past dues should be excluded from portfolio size for granularity : Basel2 - 76 */ 
  /*          TOTAL_RM_EXPOSURE_STD = sum(TOTAL_RM_EXPOSURE_STD, max(EXP_VALUE,0));*/
    
  /*  if last then do;
        /*- create macro variable for total exposure for residential mortgage retail portfolio -*/
  /*      call symput('TOTAL_RM_EXPOSURE_STD', TOTAL_RM_EXPOSURE_STD);
    end;
    run;

/*- 2: get residential mortgage portfolio size under STD and size for each counterparty -*/
 /*   data cpty_retail_rme
    (keep=COUNTERPARTY_RK TOTAL_RM_CPTY_STD );
    set _all_retail_rme(keep=EXP_VALUE COUNTERPARTY_RK APPROACH STD_PRODUCT_CLASS STD_PRODUCT_SUBCLASS IRB_PRODUCT_CLASS
              PAST_DUE_CLAIM_FLG IN_DEFAULT_FLG IRB_PRODUCT_SUBCLASS);
    by COUNTERPARTY_RK;
    
    retain TOTAL_RM_CPTY_STD 0;

    if first.COUNTERPARTY_RK then do;

        TOTAL_RM_CPTY_STD = 0 ;
    end;

    if APPROACH = 'STD' then do;
        /*- total residential mortgage exposure under STD for every counterparty -*/
 /*           if (STD_PRODUCT_CLASS = 'MORTGAGE' and STD_PRODUCT_SUBCLASS ='RESIDENTIAL') 
           then TOTAL_RM_CPTY_STD = sum(TOTAL_RM_CPTY_STD, EXP_VALUE);   
     end;
    
    if last.COUNTERPARTY_RK then output;
    
    run;
   
    
/*- 3 Part2: retail portfolio qualifying criteria check for Residential Mortgage -*/
 /*   data &RESLIB1..retail_granularity_exception_rme
         (keep=EXP_ID COUNTERPARTY_RK APPROACH COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS EXT_ISSUE_MAP_RATING 
              STD_RW_BUCKET IRB_PRODUCT_CLASS EXT_MAP_RATING UNSECURED_RM_RETAIL_CRITERIA
         index=(EXP_ID)
         );    
    merge _all_retail_rme cpty_retail_rme;
    by COUNTERPARTY_RK;
    
/*- qualifying criteria for regulatory retail portfolio under STD approach -*/
/*%if "&USE_UNDRAWN_RETAIL_GRAN_CHECK" = "YES" %then %do ;
    if FACILITY_FLG ne 1 ;
%end ;

    if APPROACH = 'STD' then do;
       if (STD_PRODUCT_CLASS = 'MORTGAGE' and STD_PRODUCT_SUBCLASS = 'RESIDENTIAL') then do;
            if (TOTAL_RM_CPTY_STD > ((&STD_RETAIL_CPTY_REL_LIMIT / 100) *  (&TOTAL_RM_EXPOSURE_STD + 0))) or 
               (TOTAL_RM_CPTY_STD > &STD_RETAIL_CPTY_ABS_LIMIT) then do;
                              
                              UNSECURED_RM_RETAIL_CRITERIA = "N" ;        
               end ;
               else do ;                  
                           
                           UNSECURED_RM_RETAIL_CRITERIA = "Y" ;        
               end ;       

        end;
    end;
run;    

  data &RESLIB1..all_exposures;
    merge &RESLIB1..all_exposures(in=exp)
           &RESLIB1..retail_granularity_exception_rme(keep=EXP_ID COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS 
                     IRB_PRODUCT_CLASS STD_RW_BUCKET APPROACH UNSECURED_RM_RETAIL_CRITERIA in=retail)
              ;
        by EXP_ID;
        run;
   
   
%end;    
        
/*- 5: reporting -*/
        %put ;
        %print_msg(warning,2,50);
        %print_msg(warning,2,51);
        %print_msg(warning,2,52);
        %put ;

    %end;

    %EXIT :   

%mend r00_post_mtm_6_crd4;
